import pinyin from "pinyin";

export class TSVector {
	#items: string[];


	private constructor(items: string[]) {
		this.#items = items;
	}

	/**
	 * 数据库值
	 */
	public get value() {
		return this.#items.join(" ");
	}

	/**
	 * 检测是否包含指定元素
	 * @param item 元素
	 */
	public contains(item: string) { return this.#items.includes(item); }

	/** 获取元素列表 */
	public get items() { return this.#items; }

	/**
	 * 添加元素
	 * @param item 要添加的元素
	 */
	public push(item: string) {
		item = item.replaceAll("'", "");
		if (this.contains(item)) return;
		this.#items.push(item);
	}

	/**
	 * 删除元素
	 * @param item 要删除的元素
	 */
	public remove(item: string) {
		item = item.replaceAll("'", "");
		const index = this.#items.indexOf(item);
		if (index >= 0) this.#items.splice(index, 1);
	}

	/**
	 * 从数据库值构建
	 * @param value 数据库值
	 */
	public static fromValue(value: string) {
		const items = value.split(" ").map(s => s.trim()).filter(s => !!s).map(s => s.replaceAll("'", ""));
		return new this(items);
	}

	/**
	 * 从关键词构建
	 * @param keywords 关键词列表
	 */
	public static fromKeyworlds(keywords: string[]) {
		const items: string[] = []

		function resolvePinyin(pinyins: string[][]): string[][] {
			if (!pinyins.length) return [];
			let [first, ...rest] = pinyins;
			//后续结果
			const ties = resolvePinyin(rest);
			//处理第一个
			first = first.map(s => s.trim().replace(/[^a-zA-Z0-9 ]/g, "")).filter(s => !!s);
			if (!first.length) return ties;

			//组合结果
			const result: string[][] = [];
			for (let py of first) {
				let leads = py.split(" ");		//这里处理一下英文和数字
				if (ties.length) {
					for (const tie of ties) result.push([...leads, ...tie]);
				}
				else result.push(leads);
			}

			//返回
			return result;
		}

		//处理关键词
		keywords = keywords.map(s => s.trim()).filter(s => !!s);

		//屏蔽纯字母数字的关键字
		keywords = keywords.filter(kw => {
			if (/^[a-zA-Z0-9]+$/.test(kw)) {
				items.push(kw);
				return false;
			}
			return true;
		})

		//处理给定的每个关键词
		for (let kw of keywords) {
			//获取拼音
			const pinyinList = pinyin(kw, { style: pinyin.STYLE_NORMAL, segment: true, heteronym: true });

			//处理拼音
			for (const pys of resolvePinyin(pinyinList)) {
				//基础拼音
				const basePinyin = pys.join("");
				if (!items.includes(basePinyin)) items.push(basePinyin);
				//首字母
				const simplePinyin = pys.map(p => {
					if (/^\d+$/.test(p)) return p;
					return p[0];
				}).join("");
				if (!items.includes(simplePinyin)) items.push(simplePinyin);
			}

			//关键字本身
			kw = kw.replaceAll(" ", '');
			if (!items.includes(kw)) items.push(kw);
		}

		//完成
		return new this(items.map(s => s.replaceAll("'", "")));
	}
}


// console.log(TSVector.fromKeyworlds(["yizhi kangkang"]))